// A program to display a parametric surface
//
// Use 'l' to render the mesh
// Use 's' to render the surface

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut.h>
#include "Trackball.h"
#include "Lighting.h"
#include "Geometry.h"

const int windowWidth=400;
const int windowHeight=400;

typedef enum {LINE_MODE, SURFACE_MODE} RENDER_OPTION;
RENDER_OPTION current_render_option=SURFACE_MODE;


// material properties 
GLfloat mat_specular[4];
GLfloat mat_ambient_and_diffuse[4];
GLfloat mat_shininess[1];

CTrackball trackball;
CLighting lighting;


void handleMouseMotion(int x, int y)
{	
	trackball.tbMotion(x, y);
}

void handleMouseClick(int button, int state, int x, int y)
{
	trackball.tbMouse(button, state, x, y);
}

void handleKeyboardEvent(unsigned char key, int x, int y)
{
	switch (key)
	{
		case 'l': current_render_option=LINE_MODE; 
					glutPostRedisplay(); break;
		case 's': current_render_option=SURFACE_MODE; 
					glutPostRedisplay(); break;
		default: trackball.tbKeyboard(key);
	}
}

void display(void)
{
	glMatrixMode( GL_MODELVIEW );	// Set the view matrix ...
	glLoadIdentity();				// ... to identity.
    gluLookAt(0,0,6, 0,0,0, 0,1,0); // camera is on the z-axis
	trackball.tbMatrix();			// rotate scene using the trackball

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	int numSegmentsS=10;
	int numSegmentsT=30;

	glEnable(GL_NORMALIZE);
	glFrontFace(GL_CW);
	if (current_render_option==LINE_MODE)
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	else
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glTranslatef(-0.8,-0.5,0);
	glScalef(2,2,2);
	int i,j;
	float s,t;
	CVec3df v1,v2,n;
	for(i=0;i<numSegmentsS;i++)
	{
		glBegin(GL_QUAD_STRIP);
		for(j=0;j<=numSegmentsT;j++)
		{	
			s=(float) i/(float) numSegmentsS;
			t=(float) j/(float) numSegmentsT;
			v1.setVector(1,0,0.2*s*2*Pi*cos(2*Pi*t));	
			v2.setVector(0,1,0.2*sin(2*Pi*t));	
			n=cross(v1,v2); 
			n.normaliseDestructive();
			glNormal3fv(n.getArray());
			glVertex3f(t,s,0.2*sin(2*Pi*t)*s);

			s=(float) (i+1)/(float) numSegmentsS;
			v1.setVector(1,0,0.2*s*2*Pi*cos(2*Pi*t));	
			v2.setVector(0,1,0.2*sin(2*Pi*t));	
			n=cross(v1,v2); 
			n.normaliseDestructive();
			glNormal3fv(n.getArray());
			glVertex3f(t,s,0.2*sin(2*Pi*t)*s);
		}	
		glEnd();
	}

	glFlush ();
	glutSwapBuffers();
}

void init(void) 
{
	// select clearing color (for glClear)
	glClearColor (1,1,1,1);	// RGB-value for black
	// enable depth buffering
	glEnable(GL_DEPTH_TEST);
	// initialize view (simple orthographic projection)
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(33,1,2,8);
	trackball.tbInit(GLUT_LEFT_BUTTON);

	// material properties 
	mat_ambient_and_diffuse[0]=0.8;		// red material ...
	mat_ambient_and_diffuse[1]=0.0;
	mat_ambient_and_diffuse[2]=0.0;
	mat_ambient_and_diffuse[3]=1;
	mat_specular[0]=0.8f;				// ... with white highlights
	mat_specular[1]=0.8f;				// if light source is reflected
	mat_specular[2]=0.8f;				// on the material surface.
	mat_specular[3]=1;
	mat_shininess[0]=100;
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_ambient_and_diffuse);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);

	// enable shading and lighting
	lighting.enable();
	glShadeModel(GL_SMOOTH);
}

void reshape(int width, int height ) {
	// Called at start, and whenever user resizes component
	int size = min(width, height);
	glViewport(0, 0, size, size);  // Largest possible square
	trackball.tbReshape(width, height);
}


// create a double buffered colour window
int main(int argc, char** argv)
{
	printf("\nUse 'l' to render the mesh");
	printf("\nUse 's' to render the surface");
	printf("\n");

	glutInit(&argc, argv);		
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(windowWidth, windowHeight); 
	glutInitWindowPosition(100, 100);
	glutCreateWindow("Parametric Surface");
	init ();								// initialise view
	glutMouseFunc(handleMouseClick);		// Set function to handle mouse clicks
	glutMotionFunc(handleMouseMotion);		// Set function to handle mouse motion
	glutKeyboardFunc(handleKeyboardEvent);	// Set function to handle keyboard input
	glutDisplayFunc(display);		// Set function to draw scene
	glutReshapeFunc(reshape);		// Set function called if window gets resized
	glutMainLoop();
	return 0;
}
